unit Graphker;

interface

uses windows, SysUtils, Graphics, Classes, Printers,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
   system.UITypes, system.Generics.Collections,
   math, maths, constantes;

const
   IntSansSignif   = -1;
   MaxCourbes      = 5;
   CouleurTitre    = clBlack;
   LongNombreAxe   = 4;
   NbreGraduations = 10;
   NbreGradMin     = 5;
   CouleurAxe: TColor = clBlack;
   CouleurCourbe: Tcolor = clNavy;
   CouleurRepere: Tcolor = clRed;
   CouleurDerivee: array[1..3] of Tcolor = (clGreen,clTeal,clMoneyGreen);

type
   TEtatSpectre = (esBlancEnCours, esSpectreEnCours, esAttente, esLectureAbs, esEcriture);
   Enombre      = class(Exception);

   Trepere = record
      Valeur: double;
      Text:   string;
   end;

   Tgraphe = class;

   Tdonnees = record
        X,A,T : Tvecteur;
        nombre : integer;
   end;

   Tmonde = class
      mini, maxi: double;
      valeur : Tvecteur;
      a, b:       double; // entier = rel*a+b
      zeroInclus: boolean;
      // Trans = Transmittance
      avecAxe, grandAxe: boolean;
      nom,nomTrans:   String;
      unite:   String;
      signif:     String;
      deltaAxe, pointDebut: double;
      // deltaAxe = cart entre graduations sur les axes
      constructor Create;
      procedure setDelta;
      procedure affecteEntier(r: double; var i: integer);
      function FormatAxe(x: double): String;
   end;

   Tgraphe = class
      nbrePoints:     indiceVecteur;
      listeDonnees : TList<Tdonnees>;
      listeRepere:  TList<TRepere>;
      mondeX,mondeAbs,MondeTrans,mondeY: Tmonde;
      mondeDer: array[1..3] of Tmonde;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      quadrillage: boolean;
      canvas:     Tcanvas;
      Titre: String;
      traceDer : array[1..3] of boolean;
      hauteurSpectre : integer;
      largeurEcran : integer;
      dataSaved : boolean;
      ImprimanteMonochrome: boolean;
      penWidth : integer;
      constructor Create;
      destructor Destroy; override;
      procedure Draw(Acanvas : Tcanvas);
      procedure TraceLigne;
      procedure TraceOld(i : integer);
      procedure TraceDerivee(j : integer);
      procedure Reset;
      procedure WindowXY(xr, yr: double;monde : Tmonde; var xi, yi: integer);
      procedure MondeXY(xi, yi: integer;monde : Tmonde; var xr, yr: double);
      procedure ChercheMonde;
      procedure SetMonde(minX, minY, maxX, maxY: double);
      procedure DrawAxis;
      procedure TraceUnPoint(i: indiceVecteur);
      procedure SetHauteurFonte(H: integer);
      procedure Imprime;
      function CourbeProche(xS,yS : integer) : integer;
      procedure AddCourbeCourante;
      procedure TraceRepere(repere: Trepere);
      procedure AjouteRepere(V: double; const T: string);
   end;

implementation

function CouleurOld(i: integer): Tcolor;
begin
     i := i mod 4;
     case i of
          0 : result := clBlue;
          1 : result := clAqua;
          2 : result := clSkyBlue;
          else result := clNavy;
     end;
end;

function GetCouleurPale(color1: Tcolor): Tcolor;
var
  R, G, B: byte;
begin
  case color1 of
    clBlack: Result := clGray;
    clMaroon: Result := clPurple;
    clGreen: Result := clTeal;
    clOlive: Result := clYellow;
    clNavy: Result := clBlue;
    clPurple: Result := clRed;
    clTeal: Result := clLime;
    clGray: Result := clMedGray;
    clRed: Result  := clFuchsia;
    clLime: Result := clMoneyGreen;
    clYellow: Result := clCream;
    clBlue: Result := clAqua;
    clAqua: Result := clSkyBlue;
    clWhite: Result := clWhite;
    clMedGray: Result := clSilver;
    else begin
      B := getBvalue(color1);
      if B < 200
         then B := B + 25
         else B := 225;
      R := getRvalue(color1);
      if R < 200
         then R := R + 25
         else R := 225;
      G := getGvalue(color1);
      if G < 200
         then G := G + 25
         else G := 225;
      Result := RGB(R, G, B);
    end;
  end;
end;

Function WavelengthToRGB(CONST Wavelength:  double) : Tcolor;
  CONST
    Gamma        =   0.80;
    IntensityMax = 255;
  VAR
    Blue   :  DOUBLE;
    factor :  DOUBLE;
    Green  :  DOUBLE;
    Red    :  DOUBLE;
    R,G,B:  BYTE;
    L : integer;

  FUNCTION Adjust(CONST Color, Factor:  DOUBLE):  INTEGER;
  BEGIN
    IF Color = 0.0
    THEN RESULT := 0     // Don't want 0^x = 1 for x <> 0
    ELSE RESULT := ROUND(IntensityMax * Power(Color * Factor, Gamma))
  END {Adjust};

BEGIN
  if WaveLength>1
     then L := round(WaveLength) // en nm
     else if WaveLength<1e-6
        then L := round(WaveLength*1e9) // en m
        else if WaveLength<1
           then L := round(WaveLength*1e6) // en m
           else L := 0;
  CASE L OF
    380..439: BEGIN
        Red   := -(L - 440) / (440 - 380);
        Green := 0.0;
        Blue  := 1.0
      END;
    440..489: BEGIN
        Red   := 0.0;
        Green := (L - 440) / (490 - 440);
        Blue  := 1.0
      END;
    490..509: BEGIN
        Red   := 0.0;
        Green := 1.0;
        Blue  := -(L - 510) / (510 - 490)
      END;
    510..579: BEGIN
        Red   := (L - 510) / (580 - 510);
        Green := 1.0;
        Blue  := 0.0
      END;
    580..644: BEGIN
        Red   := 1.0;
        Green := -(L - 645) / (645 - 580);
        Blue  := 0.0
      END;
    645..780: BEGIN
        Red   := 1.0;
        Green := 0.0;
        Blue  := 0.0
      END;
    ELSE Begin
      Red   := 0.0;
      Green := 0.0;
      Blue  := 0.0
    end;
  END;
  // Let the intensity fall off near the vision limits
  CASE L OF
    380..419:  factor := 0.3 + 0.7*(L - 380) / (420 - 380);
    420..700:  factor := 1.0;
    701..780:  factor := 0.3 + 0.7*(780 - L) / (780 - 700)
    ELSE       factor := 0.0
  END;
  R := Adjust(Red,   Factor);
  G := Adjust(Green, Factor);
  B := Adjust(Blue,  Factor);
  result := RGB(R,G,B);
END; // WavelengthToRGB

constructor Tmonde.Create;
begin
   inherited Create;
   A := 1;
   B := 0;
   nom := '';
   unite := '';
   signif := '';
   ZeroInclus := True;
   Maxi := 255;
   Mini := 0;
end;// Tmonde.Create

procedure Tmonde.affecteEntier(r: double; var i: integer);
begin
      i := round(A * r + B);
end;

procedure TGraphe.MondeXY(xi, yi: integer;monde : Tmonde;var xr, yr: double);
begin
   xr := (xi - mondeX.B) / mondeX.A;
   yr := (yi - monde.B) / monde.A;
end;

procedure Tgraphe.SetHauteurFonte(H: integer);
begin
   with canvas.Font do begin
      if H <= 13 then
         H := 13 { 8 points = 13 pixels }
      else if H <= 18 then
         H := 16 { 10 points = 16 pixels }
      else if H <= 24 then
         H := 20; { 12 points = 20 pixels }
      Name := 'Calibri';
      Height := H;
      HautCarac := H;
      Marge := hautCarac div 3;
      LargCarac := 2 * H div 3;
   end;
end;

procedure TGraphe.WindowXY(xr, yr: double;monde : Tmonde; var xi, yi: integer);
begin
   try
      mondeX.affecteEntier(xr, xi);
      monde.affecteEntier(yr, yi);
   except
      xi := IntSansSignif;
      yi := IntSansSignif
   end;
end;

procedure TGraphe.ChercheMonde;

   procedure chercheProjectionY;
   begin with mondeY do begin
         A := (limiteCourbe.top - limiteCourbe.bottom) / (Maxi - Mini);
         B := limiteCourbe.bottom - Mini * A;
         setDelta;
   end end; // chercheProjectionY

   procedure chercheProjectionDerivee(i : integer);
   begin with mondeDer[i] do begin
         GetMinMax(valeur,NbrePoints,mini,maxi);
         if (mini*maxi>0) and (abs(mini)<abs(maxi)/10) then mini := 0;
         if mini=maxi then maxi := mini+1;
         A := (limiteCourbe.top - limiteCourbe.bottom) / (Maxi - Mini);
         B := limiteCourbe.bottom - Mini * A;
         setDelta;
   end end; // chercheProjectionDerivee

   procedure chercheProjectionX;
   begin with mondeX do begin
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta;
   end end;

var i : integer;
    TraceUneDerivee : boolean;
begin // ChercheMonde
   with limiteFenetre do begin
      mondeY.avecAxe := (bottom - top) > NbreGradMin * hautCarac * 5 div 4;
      mondeX.avecAxe := (right - left) > NbreGradMin * 4 * largCarac;
      mondeY.grandAxe := (bottom - top) > NbreGraduations * hautCarac * 5 div 4;
      mondeX.grandAxe := (right - left) > NbreGraduations * 4 * largCarac;
   end;
   limiteCourbe := limiteFenetre;
   Inc(limiteCourbe.left, marge);
   Inc(limiteCourbe.top, marge);
   hauteurSpectre := LimiteFenetre.Height div 50;
   dec(limiteCourbe.bottom, marge);
   if mondeX.nom<>'t' then dec(limiteCourbe.bottom, hauteurSpectre);
   Dec(limiteCourbe.right, marge);
   if mondeY.avecAxe then
      Inc(limiteCourbe.left, LongNombreAxe * largCarac);
   if mondeX.avecAxe then
      Dec(limiteCourbe.bottom, hautCarac);
   chercheProjectionX;
   chercheProjectionY;
   TraceUneDerivee := false;
   for i := 1 to 3 do traceUneDerivee := traceUneDerivee or traceDer[i];
   if traceUneDerivee and (NbrePoints>NbrePointsDerivee) then
       DeriveeGauss(mondeX.valeur,mondeY.valeur,NbrePoints,
          mondeDer[1].valeur,mondeDer[2].valeur,mondeDer[3].valeur);
   for i := 1 to 3 do if traceDer[i] and (NbrePoints>NbrePointsDerivee) then
       chercheProjectionDerivee(i);
end;// ChercheMonde

procedure TGraphe.SetMonde(minX, minY, maxX, maxY: double);
begin
   mondeX.mini := minx;
   mondeX.maxi := maxx;
   mondeAbs.mini := miny;
   mondeAbs.maxi := maxy;
   mondeTrans.maxi := power(10,-minY);
   mondeTrans.mini := power(10,-maxY);
   if mondeTrans.mini<mondeTrans.maxi/10 then
      mondeTrans.mini := 0;
end;// SetMonde

procedure Tgraphe.TraceUnPoint(i: indiceVecteur);
var
   xi, yi: integer;
begin
   if i=0 then exit;
   canvas.pen.color := CouleurCourbe;
   if imprimanteMonochrome then canvas.pen.color := clBlack;
   canvas.pen.style := psSolid;
   WindowXY(mondeX.valeur[i], mondeY.valeur[i], mondeY, xi, yi);
   canvas.moveTo(xi, yi);
   WindowXY(mondeX.valeur[pred(i)], mondeY.valeur[pred(i)], mondeY, xi, yi);
   canvas.lineTo(xi, yi);
end; // traceUnPoint

procedure TGraphe.TraceLigne;
var
   i: indiceVecteur;
   xi, yi: integer;
begin
      if NbrePoints = 0 then exit;
      canvas.pen.style := psSolid;
      canvas.pen.color := couleurCourbe;
      if imprimanteMonochrome then canvas.pen.color := clBlack;
      WindowXY(mondeX.valeur[0], mondeY.valeur[0], mondeY, xi, yi);
      canvas.MoveTo(xi, yi);
      for i := 1 to pred(NbrePoints) do begin
         WindowXY(mondeX.valeur[i], mondeY.valeur[i], mondeY, xi, yi);
         canvas.LineTo(xi, yi)
      end;
end;// TraceLigne

procedure TGraphe.TraceOld(i : integer);
var
   j: indiceVecteur;
   xi, yi: integer;
begin with listeDonnees[i] do begin
      canvas.pen.style := psSolid;
      canvas.pen.color := couleurOld(i);
      if mondeY=mondeAbs then begin
      WindowXY(X[0], A[0], mondeY, xi, yi);
      canvas.MoveTo(xi, yi);
      for j := 1 to pred(Nombre) do begin
         WindowXY(X[j], A[j], mondeY, xi, yi);
         canvas.LineTo(xi, yi)
      end;
      end
      else begin
      WindowXY(X[0], T[0], mondeY, xi, yi);
      canvas.MoveTo(xi, yi);
      for j := 1 to pred(Nombre) do begin
         WindowXY(X[j], T[j], mondeY, xi, yi);
         canvas.LineTo(xi, yi)
      end;
      end;
end end;// TraceOld

procedure TGraphe.TraceDerivee(j : integer);
var
   i: indiceVecteur;
   xi, yi: integer;
begin
      if NbrePoints<=NbrePointsDerivee then exit;
      canvas.pen.style := psSolid;
      canvas.pen.color := couleurDerivee[j];
      if imprimanteMonochrome then canvas.pen.color := clBlack;
      WindowXY(mondeX.valeur[0], mondeDer[j].valeur[0], mondeDer[j], xi, yi);
      canvas.MoveTo(xi, yi);
      for i := 1 to pred(NbrePoints) do begin
         WindowXY(mondeX.valeur[i], mondeDer[j].valeur[i], mondeDer[j], xi, yi);
         canvas.LineTo(xi, yi)
      end;
end;// TraceDerivee

procedure TGraphe.Draw(Acanvas : TCanvas);
var i : integer;
    margeImpr : integer;
begin
      Canvas := ACanvas;
      LimiteFenetre := canvas.clipRect;
      if LimiteFenetre.right>largeurEcran then begin // Imprimante ?
         margeImpr := LimiteFenetre.Bottom div 64;
         inc(limiteFenetre.Top,margeImpr);
         limiteFenetre.Bottom := limiteFenetre.Bottom div 2;
         inc(limiteFenetre.left,margeImpr);
         dec(limiteFenetre.right,margeImpr);
         PenWidth := (2*limiteFenetre.Right div largeurEcran);
         if penWidth<1 then Penwidth := 1;
      end
      else Penwidth := 1;
      with limiteFenetre do
           setHauteurFonte((bottom - top) div 30);
      if LimiteFenetre.right<largeurEcran then begin
         canvas.pen.color := clWhite;
         canvas.brush.style := bsSolid;
         canvas.brush.color := clWhite;
         canvas.FillRect(LimiteFenetre);
      end;
      chercheMonde;
      drawAxis;
      canvas.Pen.Width := penWidth;
      traceLigne;
      for i := 1 to 3 do if traceDer[i] then
          traceDerivee(i);
      for i := 0 to listeDonnees.Count-1 do
          traceOld(i);
      for i := 0 to pred(ListeRepere.Count) do
         TraceRepere(ListeRepere[i]);
end;// Draw

procedure TGraphe.Reset;
begin
   NbrePoints := 0;
   DataSaved := false;
   ListeRepere.Clear;
end;

constructor TGraphe.Create;
var i : integer;
begin
   inherited Create;
   ImprimanteMonochrome := false;
   titre := '';
   mondeX  := Tmonde.Create;
   mondeX.zeroInclus := false;
   mondeAbs  := Tmonde.Create;
   mondeTrans  := Tmonde.Create;
   mondeY := mondeAbs;
   quadrillage := True;
   NbrePoints := 0;
   for i := 1 to 3 do
       mondeDer[i] := Tmonde.Create;
   listeDonnees := Tlist<Tdonnees>.create;
   listeRepere := TList<TRepere>.Create;
end;

destructor TGraphe.Destroy;
var i : integer;
begin
   mondeX.Free;
   mondeAbs.Free;
   mondeTrans.Free;
   for i := 1 to 3 do
       mondeDer[i].free;
   listeDonnees.free;
   listeRepere.Free;
   inherited Destroy;
end;

procedure Tmonde.SetDelta;
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   largeur: double;
   Exponent: integer;
   index: byte;
begin
   Exponent := floor(Log10(Maxi - Mini));
   largeur := (maxi - mini) * power(10,-exponent);
   index := 0;
   if grandAxe
   then while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * power(10,exponent);
   PointDebut := ceil(mini / deltaAxe) * deltaAxe;
end;

function Tmonde.FormatAxe(x: double): String;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, 3, 0);
end;

procedure Tgraphe.Imprime;
begin
   if MessageDlg('Impression ?', mtConfirmation, mbOKCancel, 0) = mrOk then begin
      ImprimanteMonochrome := GetDeviceCaps(printer.Handle,NumColors) <= 2;
      printer.Orientation := poPortrait;
      printer.beginDoc;
      draw(printer.canvas);
      ImprimanteMonochrome := false;
      printer.endDoc;
   end;
end;

procedure TGraphe.DrawAxis;
var xAxeY,yAxeX : integer;
    xNombreMax,signeMargeX : integer;
    PointCourant : single;

procedure TraceCouleursSpectre;
var
    i, i0:  integer;
    xi, yi: double;
    y1, y2: integer;
    k, kmax, pas: integer;
    couleurT: Tcolor;
begin
        i0 := limiteCourbe.left;
        kMax := limiteCourbe.right - limiteCourbe.left;
        y1 := limiteFenetre.bottom;
        y2 := y1-hauteurSpectre;
        pas := succ(kmax div 256);
        kmax := kmax div pas;
        if pas=1 then begin
           canvas.pen.style := psSolid;
        end
        else begin
          canvas.Pen.style := psClear;
          canvas.Brush.style := bsSolid;
          canvas.brush.color := clWhite;
        end;
        for k := 0 to kmax do begin
          i := i0 + pas * k;
          mondeXY(i, 0, mondeY, xi, yi);
          couleurT := wavelengthToRGB(xi);
          if pas = 1 then begin
            canvas.pen.color := couleurT;
            canvas.moveto(i, y1);
            canvas.lineTo(i, y2);
          end
          else begin
            canvas.brush.color := couleurT;
            canvas.rectangle(i + pas, y1, i - pas, y2);
          end;
        end; // for k
end; // traceCouleursSpectre

procedure TitreAxeX;
var xTitre,yTitre : integer;
    prompt : string;
begin
        yTitre := limiteCourbe.bottom+marge;
        with mondeX do if unite=''
             then prompt := nom
             else prompt := nom+'('+unite+')';
        xTitre := LimiteFenetre.right-canvas.textWidth(prompt);
        canvas.TextOut(xTitre,yTitre,prompt);
    	  xNombreMax := xTitre-(largCarac*longNombreAxe div 2);
end;

procedure TitreAxeY;
var prompt : string;
begin
      with mondeY do if unite<>''
          then prompt := nom+'('+unite+')'
          else prompt := nom;
      canvas.textOut(limiteFenetre.left+marge,limiteFenetre.Top,prompt);
end;

Function YcorrectNombre(y : integer) : boolean;
var limiteTop : integer;
begin with mondeY do begin
     LimiteTop := limiteCourbe.Top+marge;
     YcorrectNombre := ( y>(LimiteTop)) and
                       ( y<(limiteCourbe.Bottom-marge)) and
		       ( abs(y-yAxeX)>marge )
end end;

Function YcorrectGrad(y : integer) : boolean;
begin with mondeY do
     YcorrectGrad := ( y>limiteCourbe.Top) and
                     ( y<limiteCourbe.Bottom)
end;

Function XcorrectNombre(x : integer) : boolean;
begin
     XcorrectNombre := ( x>(limiteCourbe.left+marge)) and
                       ( x<xNombreMax) and
                       ( abs(x-xAxeY)>marge )
end;

Function XcorrectGrad(x : integer) : boolean;
begin
     XcorrectGrad := ( x>limiteCourbe.left) and
                     ( x<limiteCourbe.right)
end;

procedure OutTextY(y : integer;S : string);
begin
   canvas.TextOut(limiteFenetre.left+marge,y-hautCarac div 2,S)
end;

procedure OutTextX(x : integer;S : string);
begin
   canvas.TextOut(x,yAxeX+marge,S)
end;

procedure MyLine(x1,y1,x2,y2 : integer);
begin
     canvas.moveTo(x1,y1);
     canvas.lineTo(x2,y2);
end;

Procedure graduationX(x : integer;quadrillage : boolean);
begin
  if quadrillage
       then MyLine(x,limiteCourbe.top,x,limiteCourbe.bottom)
       else MyLine(x,yAxeX,x,yAxeX+marge);
end;

Procedure graduationY(y : integer;quadrillage : boolean);
begin
   if quadrillage
	 then MyLine(limiteCourbe.left,y,limiteCourbe.right,y)
	 else MyLine(xAxeY,y,xAxeY-signeMargeX*marge,y);
end;

  procedure initAxeX;
  var x1,y1 : integer;
  begin	 with mondeX,canvas do begin
	 PointCourant := PointDebut;
	 WindowXY(0,0,mondeY,x1,y1);
	 if (x1>limiteCourbe.left) and (x1<limiteCourbe.right) then begin
		 pen.style := psDot;
		 Myline(x1,limiteCourbe.bottom,x1,limiteCourbe.top);
                 pen.style := psSolid;
	 end;
     TitreAxeX;
	   MyLine(limiteFenetre.left,yAxeX,limiteCourbe.right,yAxeX);
                    { flches }
     MyLine(LimiteCourbe.right-marge*3,yAxeX+marge,LimiteCourbe.right,yAxeX);
     MyLine(LimiteCourbe.right-marge*3,yAxeX-marge,LimiteCourbe.right,yAxeX);
  end end;// initAxeX

  procedure traceAxeX;
  var xs,ys : integer;
      NombreAff : String;
  begin with mondeX do begin
      initAxeX;
      if Quadrillage then canvas.pen.style := psDot;
      repeat
          windowXY(PointCourant,mondeY.Mini,mondeY,xs,ys);
      	  if XcorrectGrad(xs) then begin
               graduationX(xs,Quadrillage);
               if XcorrectNombre(xs) then begin
         	        if abs(PointCourant)< (Maxi-Mini)/100 then pointCourant := 0;
                  NombreAff := formatAxe(pointCourant);
        	        OutTextX(xs,NombreAff);
      	       end;
          end;
      	  PointCourant := PointCourant+deltaAxe;
      until ( PointCourant>=Maxi );
      canvas.pen.style := psSolid;
 end end;// traceAxeX

 procedure initAxeY;
 var x1,y1 : integer;
 begin	with mondeY do begin
	PointCourant := PointDebut;
        WindowXY(0,0,mondeY,x1,y1);
        if (y1>limiteCourbe.Top) and (y1<limiteCourbe.Bottom)then begin
            canvas.pen.style := psDot;
	          Myline(limiteCourbe.left,y1,limiteCourbe.right,y1);
            canvas.pen.style := psSolid;
        end;
        MyLine(xAxeY,yAxeX,xAxeY,limiteCourbe.Top);
              { flches }
        MyLine(xAxeY-marge,LimiteCourbe.Top+3*marge,xAxeY,LimiteCourbe.Top);
        MyLine(xAxeY+marge,LimiteCourbe.Top+3*marge,xAxeY,LimiteCourbe.Top);
        TitreAxeY;
 end end;// initAxeY

procedure traceTitre;
var largeur : integer;
begin
      if titre='' then exit;
      canvas.pen.color := couleurTitre;
      canvas.font.color := couleurTitre;
      if imprimanteMonochrome then canvas.pen.color := clBlack;
      if imprimanteMonochrome then canvas.font.color := clBlack;
      canvas.font.style := [fsBold];
      setTextAlign(canvas.handle,TA_left+TA_top);
      largeur := limiteCourbe.right div canvas.TextWidth('A');
      canvas.TextOut(limiteCourbe.left+marge,LimiteFenetre.Top+marge,
         copy(Titre,1,largeur));
      canvas.font.style := [];
end;// traceTitre

 procedure traceAxeY;
 var xs,ys : integer;
     nombreAff : String;
 begin with mondeY do begin
     initAxeY;
     if Quadrillage then canvas.pen.style := psDot;
     repeat
           windowXY(mondeX.Mini,PointCourant,mondeY,xs,ys);
 // coordonne cran de marque
	   if YcorrectGrad(ys) then begin
	   	 graduationY(ys,quadrillage);
         	 if YcorrectNombre(ys) then begin
	   	        if abs(pointCourant) < (Maxi-Mini)/1000 then pointCourant := 0;
                    NombreAff := formatAxe(pointCourant);
	   	        OutTextY(ys,NombreAff);
           end;
	   end;
	   pointCourant := pointCourant+deltaAxe;
      until ( pointCourant>=Maxi );
      canvas.pen.style := psSolid;
 end end;// traceAxeY

procedure zeroX;
var x1,y1 : integer;
begin with mondeX do
	  WindowXY(0,0,mondeY,x1,y1);
	  if (x1>limiteCourbe.left) and (x1<limiteCourbe.right) then
	      Myline(x1,limiteCourbe.bottom,x1,limiteCourbe.top);
end;// zeroX

 procedure zeroY;
 var x1,y1 : integer;
 begin with mondeY do begin
 	windowXY(0,0,mondeY,x1,y1);
	if (y1>limiteCourbe.Top) and  (y1<limiteCourbe.Bottom) then
	   Myline(limiteCourbe.left,y1,limiteCourbe.right,y1);
 end end;// zeroY

begin // DrawAxis
with canvas do begin
     signeMargeX := -1;
     pen.style := psSolid;
     pen.color := couleurAxe;
     font.color := CouleurAxe;
     pen.Width := 1;
     if imprimanteMonochrome then begin
        canvas.pen.color := clBlack;
        canvas.font.color := clBlack;
     end;
     yAxeX := limiteCourbe.bottom;
     xAxeY := limiteCourbe.left;
     if mondeX.avecAxe
        then traceAxeX
        else zeroX;
     if mondeY.avecAxe
        then traceAxeY
        else zeroY;
     traceTitre;
     if mondeX.nom<>'t' then traceCouleursSpectre;
end end;// DrawAxis

function Tgraphe.CourbeProche(xS,yS : integer) : integer;
var i,j : integer;
    xi,yi : integer;
    distance,distanceMin : integer;
begin
    result := -1;
    distanceMin := 16;
    if mondeY=mondeAbs then begin
    for i := 0 to listeDonnees.count -1 do with listeDonnees[i] do begin
        for j := 0 to nombre-1 do begin
            WindowXY(X[j],A[j],mondeY,xi,yi);
            distance := abs(yS-yi)+abs(xS-Xi);
            if distance<distanceMin then begin
               distanceMin := distance;
               result := i;
            end;
        end;
    end;
    end
    else begin
     for i := 0 to listeDonnees.count -1 do with listeDonnees[i] do begin
        for j := 0 to nombre-1 do begin
            WindowXY(X[j],T[j],mondeY,xi,yi);
            distance := abs(xS-xi)+abs(yS-yi);
            if distance<distanceMin then begin
               distanceMin := distance;
               result := i;
            end;
        end;
    end;
    end;
end;

procedure Tgraphe.AddCourbeCourante;
var donnees : Tdonnees;
begin
        donnees.X := mondeX.valeur;
        donnees.A := mondeAbs.valeur;
        donnees.T := mondeTrans.valeur;
        donnees.nombre := nbrePoints;
        listeDonnees.Add(donnees);
        dataSaved := true;
end;

procedure Tgraphe.TraceRepere(repere: Trepere);
var
   x, y: integer;
begin with Canvas, repere do begin
      Pen.color := couleurRepere;
      if imprimanteMonochrome then pen.color := clBlack;
      Pen.Style := psDash;
      windowXY(valeur, 0, mondeY, x, y);
      MoveTo(x, LimiteCourbe.top);
      LineTo(x, LimiteCourbe.bottom);
      Pen.Style := psSolid;
end end;// TraceRepere

procedure Tgraphe.AjouteRepere(V: double; const T: string);
var
   repere: Trepere;
begin
   repere.valeur := V;
   repere.Text := T;
   ListeRepere.Add(repere);
   TraceRepere(repere);
end;

end.

