Unit maths;

Interface

uses Windows, SysUtils, math;

const
    MaxData = 2000;
    MaxDataCinet = 600;
    degreDerivee = 3;
    ordre = degreDerivee+1;
    DemiLargeur = 4;
    NbrePointsDerivee = 2*DemiLargeur+1;

type
    indiceVecteur = 0..MaxData;
	  Tvecteur = array[indiceVecteur] of double;

type
   matrice = array[1..ordre,1..ordre+1] of double;
   matriceLigne = array[1..ordre] of double;
   EsystemeLineaireError = class(Exception);
   vectorPuiss = array[0..2*degreDerivee] of double;

Procedure DeriveeGauss(const X,Yr : Tvecteur;N : integer;
              var Yder1,Yder2,YDer3 : Tvecteur);
Procedure GetMinMax(const V : Tvecteur;N : integer;var mini,maxi : double);

Implementation

Procedure GetMinMax(const V : Tvecteur;N : integer;var mini,maxi : double);
var i : integer;
begin
     Mini := 0;
     Maxi := 0;
     for i := 0 to pred(N) do
         if V[i]>Maxi
            then Maxi := V[i]
            else if V[i]<Mini
               then Mini := V[i]
end;


Procedure Resolution(A : matrice;ordre : integer;var Result : matriceLigne);
 { Cette procdure rsoud un systme AX = B }
 { crit sous la forme AX = A(n+1) }
 { par la mthode d'limination de Gauss (pivot partiel) }

  Procedure Elimination;

    FUNCTION Pivot(i :integer):integer;
    Var n,temp : integer;
    BEGIN
      temp := i;
      for n := i+1 to ordre do
          if abs(A[n,i]) > abs(A[i,i]) then temp:= n;
      pivot:=temp;
      if abs(A[temp,i]) <= 1e-100 then
         raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
    END;

    PROCEDURE Permute(i,j:integer);
    Var temp:double;
        k:integer;
    BEGIN
       if i<>j then For k:=i to ordre+1 do BEGIN
          temp:=A[i,k];
          A[i,k]:=A[j,k];
          A[j,k]:=temp;
       END;
    END;

  var I,J,K : integer;
      M : double;
  begin
    for k := 1 to ordre do begin
          Permute(k,pivot(k));
          for i:=k+1 to ordre do begin
              M := A[i,k]/A[k,k];
              for j:=k+1 to ordre+1 do A[i,j]:=A[i,j]-M*A[k,j];
              A[i,k]:=0.0;
          end;
       end;
    end; {Elimination}

    Procedure Substitution;
    var I,J : integer;
        S : double;
    begin
         try
         for I:=ordre downto 1 do begin
             S:=A[I,ordre+1];
             for J:=I+1 to ordre do S:=S-A[I,J]*Result[J];
             Result[I]:=S/A[I,I];
         end;
         except
            on EzeroDivide do raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
         end;
    end;

begin {Resolution}
  Elimination;
  Substitution;
end; // rsolution

procedure DeriveeGauss(const X,Yr : Tvecteur;N : integer;
   var Yder1,Yder2,Yder3 : Tvecteur);
var puissX : vectorPuiss;
    A : matrice;
    Coeff : matriceLigne;
    puissMax : integer;

procedure initXY;
var j : integer;
    loc : double;
    i : integer;
begin
    for i := 1 to puissMax do puissX[i] := 0;
    puissX[0] := NbrePointsDerivee;
    for i := 1 to ordre do A[i,ordre+1] := 0;
    for j := 0 to pred(NbrePointsDerivee) do begin
        loc := x[j];
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+loc;
            loc := loc*x[j];
        end;
        loc := yr[j];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*x[j];
        end;
    end;
end;

Procedure CalculXY(i : integer);
var ajoute,retire : double;
    k : integer;
begin
        ajoute := X[i+DemiLargeur];
        retire := X[i-DemiLargeur-1];
        for k := 1 to puissMax do begin
            puissX[k] := puissX[k]+ajoute-retire;
            retire := retire*X[i-DemiLargeur-1];
            ajoute := ajoute*X[i+DemiLargeur];
        end;
        ajoute := Yr[i+DemiLargeur];
        retire := Yr[i-DemiLargeur-1];
        for k := 1 to ordre do begin
            A[k,ordre+1] := A[k,ordre+1]+ajoute-retire;
            retire := retire*X[i-DemiLargeur-1];
            ajoute := ajoute*X[i+DemiLargeur];
        end;
end;

procedure AffecteCoeff;
var i,j : integer;
begin
     for i := 1 to ordre do
         for j := 1 to ordre do
             A[i,j] := PuissX[i+j-2];
     try
     Resolution(A,ordre,Coeff);
     except
         for j := 1 to 4 do coeff[j] := 0;
     end;
end;

Procedure AffecteDerivee(i : integer);
begin
   yder1[i] := coeff[2]+x[i]*(2*coeff[3]+3*coeff[4]*x[i]);
   yder2[i] := 2*coeff[3]+x[i]*6*coeff[4];
   yder3[i] := 6*coeff[4];
end;

var i : integer;
begin // DeriveeGauss
    for i := 0 to MaxData do begin
        yder1[i] := 0;
        yder2[i] := 0;
        yder3[i] := 0;
    end;
    if N<NbrePointsDerivee then exit;
    puissMax := 2*degreDerivee;
    initXY;
    affecteCoeff;
    for i := 0 to pred(DemiLargeur) do affecteDerivee(i);
    affecteDerivee(DemiLargeur);
    for i := DemiLargeur+1 to N-DemiLargeur-1 do begin
        calculXY(i);
        affecteCoeff;
        affecteDerivee(i);
    end;
    for i := N-DemiLargeur to pred(N) do affecteDerivee(i);
end; // DeriveeGauss

end.

